#ifndef CENTRALWIDGET_H
#define CENTRALWIDGET_H

#include <QWidget>
#include <string>
#include "StackedGraphWidget.h"
#include "DataItem.h"

/**
* This class is a Widget which contains all the loaded data and one or two stacked graph widgets.
*/
class CentralWidget : public QWidget
{
	Q_OBJECT

public:
	CentralWidget(QWidget* parent); ///< creates the central widget
	~CentralWidget();
	void loadData(const QString& filePath); ///< loads the data from the given file path
	bool hasGLAA() {return stackedGraphWidget->getStackedGraph()->getView()->hasGLAA();} ///< returns if openGL anti-aliasing is possible.

public slots:
	/**
	* \brief sets the viewport of each stacked graph view.
	* \param openGL if true, all viewports are switched to openGL redering (if possible). if false, all viewports are set to software rendering.
	*/
	void setGLViewport(bool openGL);
	/**
	* \brief sets the auto GL feature for each stacked graph view. The auto GL mode has just effects, if openGL is supported, and GL anti-aliasing is possible.
	* \param autoGL if true, activates auto GL (if possible). if false, deactivates auto GL.
	*/
	void setAutoGLViewport(bool autoGL);
	/**
	* \brief sets the detailed information feature for each stacked graph view.
	* \param detailInfo if true, activates detailed information. if false, deactivates detailed information.
	*/
	void setDetailedInformation(bool detailInfo);
	/**
	* \brief toggles the compare graph.
	* \param compareGraph if true, shows the compare graph. if false, hides the compare graph.
	*/
	void toggleCompareGraph(bool compareGraph);

private:
	/**
	* \brief deletes all occurrences of the given char from the string.
	* \param text the string which should be cleaned of all occurrences of charToDelete.
	* \param charToDelete the char which should be deleted from text.
	*/
	void cleanString(std::string& text, const char charToDelete);
	/**
	* \brief loads a DataItemPart out of the data passed in text.
	* \param text the string with the data to load.
	* \param dataDelimiter the delimiter to be used.
	* \return the DataItemPart which was loaded.
	*/
	DataItemPart stringToDataItemPart(std::string text, const char dataDelimiter);
	/**
	* \brief linearly interpolates a time series if there is one or more months missing in between two dates.
	* \param items the time series to be interpolated.
	*/
	void linearInterpolate(QVector<DataItemPart>& items);

	QVector<DataItem> data;
	QString unitSymbol;
	bool unitAfterNumber;
	QVector<QString> unitSizeNames;
	StackedGraphWidget* stackedGraphWidget;
	StackedGraphWidget* stackedGraphWidgetCompare;
};

#endif
