#include "StackedGraph.h"
#include <QPainterPath>
#include <QToolTip>
#include <float.h>
#include <math.h>

StackedGraph::StackedGraph(const QVector<DataItem>* dataPtr, const QString* unitSymbolPtr, const bool* unitAfterNumberPtr, const QVector<QString>* unitSizeNamesPtr, QWidget* parent) : QGraphicsScene(parent)
{
	data = dataPtr;
	unitSymbol = unitSymbolPtr;
	unitAfterNumber = unitAfterNumberPtr;
	unitSizeNames = unitSizeNamesPtr;
	baselineAlgo = 0;
	sortAlgo = 0;
	colorAlgo = 0;
	xScale = 1.0;
	yScale = 1.0;
	xRange = 320.0;
	yRange = 4.0;
	yMaxPos = 2.0;
	colorAlgoMinValue = 0.0;
	colorAlgoMaxValue = 0.0;
	firstYear = 0;
	firstMonth = 0;
	lastYear = 0;
	lastMonth = 1;
	renderGraph = true;
	detailedInformation = false;
	view = new StackedGraphView(this,parent);
	connect(view, SIGNAL(toolTipChanged(QGraphicsItem*,QPoint)), this, SLOT(makeToolTip(QGraphicsItem*,QPoint)));
}

StackedGraph::~StackedGraph()
{

}

void StackedGraph::makeStackedGraph()
{
	if (renderGraph) {
		this->clear();
		calculateColorAlgoMinMaxValue();
		if (!data->isEmpty()) {
			sortStackedGraph();
			paintStackedGraph();
			this->setSceneRect(0.0, -yMaxPos * yScale * 1.01, xRange, yRange * 1.01);
			//this->setSceneRect(this->itemsBoundingRect());
		}
	}
}

void StackedGraph::calculateFirstLastDate()
{
	if (!data->isEmpty() && !data->at(0).data.isEmpty()) {
		firstYear = data->at(0).data.at(0).year;
		firstMonth = data->at(0).data.at(0).month;

		lastYear = firstYear;
		lastMonth = firstMonth;
		for (int i=0; i<data->size(); i++) {
			if (getXPos(lastYear,lastMonth) < getDataXPos(i,data->at(i).data.size()-1)) {
				lastYear = data->at(i).data.at(data->at(i).data.size()-1).year;
				lastMonth = data->at(i).data.at(data->at(i).data.size()-1).month;
			}
		}
	} else {
		firstYear = 0;
		firstMonth = 0;
		lastYear = 0;
		lastMonth = 1;
		calculateXYScale();
	}
}

void StackedGraph::calculateXYScale()
{
	int graphLength = getXPos(lastYear, lastMonth);
	xScale = xRange / double(graphLength);

	QVector<double> lastHeights;
	for(int i=0; i<=graphLength; i++) {
		lastHeights.push_back(0.0);
	}
	for (int i=0; i<data->size(); i++) {
		for (int j=0; j<data->at(i).data.size(); j++) {
			int pX = getDataXPos(i,j);
			lastHeights[pX] += data->at(i).data.at(j).quantity;
		}
	}
	double maxHeight = 0.0;
	for (int i=0; i<lastHeights.size(); i++) {
		if (maxHeight < lastHeights.at(i)) {
			maxHeight = lastHeights.at(i);
		}
	}
	double smallHeight = maxHeight * 0.01;
	graphLength = 0;
	for (int i=0; i<lastHeights.size(); i++) {
		if (lastHeights.at(i) > smallHeight) {
			graphLength++;
		}
	}
	double averageHeight = 0.0;
	for (int i=0; i<lastHeights.size(); i++) {
		if (lastHeights.at(i) > smallHeight) {
			averageHeight += lastHeights.at(i) / double(graphLength);
		}
	}
	if (0.0 < averageHeight) {
		yScale = 1.0 / (averageHeight / 1000.0);
	} else {
		yScale = 1.0;
		yRange = 4.0;
		yMaxPos = 2.0;
	}
}

void StackedGraph::calculateColorAlgoMinMaxValue()
{
	if (!data->isEmpty() && !data->at(0).data.isEmpty()) {
		if (colorAlgo == 0) {
			int maxPos = getDataXPos(data->size()-1,0);
			int minPos = 0;
			colorAlgoMaxValue = maxPos;
			colorAlgoMinValue = minPos;
		} else if (colorAlgo == 1) {
			int maxPos = getXPos(lastYear,lastMonth);
			int minPos = maxPos;
			for (int h=0; h<data->size(); h++) {
				if (minPos > getDataXPos(h,data->at(h).data.size()-1)) {
					minPos = getDataXPos(h,data->at(h).data.size()-1);
				}
			}
			colorAlgoMaxValue = maxPos;
			colorAlgoMinValue = minPos;
		} else if (colorAlgo == 2) {
			double maxMaxQuantity = 0.0;
			double minMaxQuantity = DBL_MAX;
			for (int h=0; h<data->size(); h++) {
				double currentMaxQuantity = 0.0;
				for (int j=0; j<data->at(h).data.size(); j++) {
					if (currentMaxQuantity < data->at(h).data.at(j).quantity) {
						currentMaxQuantity = data->at(h).data.at(j).quantity;
					}
				}
				if (maxMaxQuantity < currentMaxQuantity) {
					maxMaxQuantity = currentMaxQuantity;
				}
				if (minMaxQuantity > currentMaxQuantity) {
					minMaxQuantity = currentMaxQuantity;
				}
			}
			colorAlgoMaxValue = maxMaxQuantity;
			colorAlgoMinValue = minMaxQuantity;
		} else if (colorAlgo == 3) {
			double maxAverageQuantity = 0.0;
			double minAverageQuantity = DBL_MAX;
			for (int h=0; h<data->size(); h++) {
				double currentAverageQuantity = 0.0;
				for (int j=0; j<data->at(h).data.size(); j++) {
					currentAverageQuantity += data->at(h).data.at(j).quantity / double(data->at(h).data.size());
				}
				if (maxAverageQuantity < currentAverageQuantity) {
					maxAverageQuantity = currentAverageQuantity;
				}
				if (minAverageQuantity > currentAverageQuantity) {
					minAverageQuantity = currentAverageQuantity;
				}
			}
			colorAlgoMaxValue = maxAverageQuantity;
			colorAlgoMinValue = minAverageQuantity;
		} else if (colorAlgo == 4) {
			double maxSumQuantity = 0.0;
			double minSumQuantity = DBL_MAX;
			for (int h=0; h<data->size(); h++) {
				double currentSumQuantity = 0.0;
				for (int j=0; j<data->at(h).data.size(); j++) {
					currentSumQuantity += data->at(h).data.at(j).quantity;
				}
				if (maxSumQuantity < currentSumQuantity) {
					maxSumQuantity = currentSumQuantity;
				}
				if (minSumQuantity > currentSumQuantity) {
					minSumQuantity = currentSumQuantity;
				}
			}
			colorAlgoMaxValue = maxSumQuantity;
			colorAlgoMinValue = minSumQuantity;
		}
	} else {
		colorAlgoMaxValue = 0.0;
		colorAlgoMinValue = 0.0;
	}
	emitColorAlgoMinMaxValue();
}

void StackedGraph::emitColorAlgoMinMaxValue()
{
	if (colorAlgo == 0 || colorAlgo == 1) {
		int maxYear = firstYear;
		int maxMonth = firstMonth;
		int minYear = firstYear;
		int minMonth = firstMonth;
		int maxGraphLength = colorAlgoMaxValue;
		for (int i=0; i<maxGraphLength; i++) {
			maxMonth++;
			if (maxMonth > 12) {
				maxYear++;
				maxMonth -= 12;
			}
		}
		int minGraphLength = colorAlgoMinValue;
		for (int i=0; i<minGraphLength; i++) {
			minMonth++;
			if (minMonth > 12) {
				minYear++;
				minMonth -= 12;
			}
		}
		emit maxColorTextChanged(QString::number(maxYear) + "."  + QString::number(maxMonth), colorScheme.getContrastColor(1.0));
		emit minColorTextChanged(QString::number(minYear) + "."  + QString::number(minMonth), colorScheme.getContrastColor(0.0));
	} else {
		emit maxColorTextChanged(addUnitToNumber(colorAlgoMaxValue), colorScheme.getContrastColor(1.0));
		emit minColorTextChanged(addUnitToNumber(colorAlgoMinValue), colorScheme.getContrastColor(0.0));
	}
}

void StackedGraph::sortStackedGraph()
{
	if (!data->isEmpty()) {
		calculateFirstLastDate();
		int graphLength = getXPos(lastYear, lastMonth);
		QVector<double> lastHeights;
		for(int i=0; i<=graphLength; i++) {
			lastHeights.push_back(0.0);
		}

		tempData.clear();
		for (int i=0; i<data->size(); i++) {
			TempData tempDataPart;
			tempDataPart.i = i;
			tempDataPart.j = -1;
			tempDataPart.item = NULL;
			tempData.push_back(tempDataPart);
		}
		int baselinePos = 0;
		bool top = true;
		for (int k=0; k<lastHeights.size(); k++) {

			for (int h=0; h<tempData.size(); h++) {
				if (-1 < tempData.at(h).j && tempData.at(h).j < data->at(tempData.at(h).i).data.size()) {
					lastHeights[k] += data->at(tempData.at(h).i).data.at(tempData.at(h).j).quantity;
				}
			}
			if (tempData.at(baselinePos).j >= data->at(tempData.at(baselinePos).i).data.size()) {
				for (int l=baselinePos+1; l<tempData.size(); l++) {
					if (-1 < tempData.at(l).j && tempData.at(l).j < data->at(tempData.at(l).i).data.size()) {
						baselinePos = l;
						break;
					}
				}
			}

			for (int i=0; i<data->size(); i++) {
				int j = 0;
				if (j < data->at(i).data.size()) {
					if (k == getDataXPos(i,j)) {
						for (int h=0; h<tempData.size(); h++) {
							if (tempData.at(h).i == i) {

								if (sortAlgo == 1 && h != baselinePos) {
									tempData[h].j = j;
									if (top) {
										tempData.push_back(tempData.at(h));
										tempData.remove(h);
										if (h < baselinePos) {
											baselinePos--;
										}
										h--;
									} else {
										tempData.push_front(tempData.at(h));
										tempData.remove(h+1);
										if (h > baselinePos) {
											baselinePos++;
										}
									}
									top = !top;
								} else if (sortAlgo == 2 && h != baselinePos) {
									double sumTopHalf = 0.0;
									int basePos = baselinePos;
									for (int m=0; m<3; m++) {
										if (tempData.at(basePos).j+m >= data->at(tempData.at(basePos).i).data.size()) {
											for (int l=basePos+1; l<tempData.size(); l++) {
												if (-1 < tempData.at(l).j && tempData.at(l).j+m < data->at(tempData.at(l).i).data.size()) {
													basePos = l;
													break;
												}
											}
										}
										double height = 0.0;
										double topHalf = 0.0;
										for (int l=0; l<tempData.size(); l++) {
											if (-1 < tempData.at(l).j && tempData.at(l).j+m < data->at(tempData.at(l).i).data.size()) {
												height += data->at(tempData.at(l).i).data.at(tempData.at(l).j+m).quantity;
												if (l == basePos) {
													topHalf += data->at(tempData.at(l).i).data.at(tempData.at(l).j+m).quantity / 2.0;
												} else if (l > basePos) {
													topHalf += data->at(tempData.at(l).i).data.at(tempData.at(l).j+m).quantity;
												}
											}
										}
										double weight = 0.5;
										if (m==1) {
											weight = 1.0;
										} else if (m>1) {
											weight = 0.25;
										}
										sumTopHalf += ((height / 2.0) - topHalf) * weight;
									}
									tempData[h].j = j;
									if (sumTopHalf >= 0) {
										tempData.push_back(tempData.at(h));
										tempData.remove(h);
										if (h < baselinePos) {
											baselinePos--;
										}
										h--;
									} else {
										tempData.push_front(tempData.at(h));
										tempData.remove(h+1);
										if (h > baselinePos) {
											baselinePos++;
										}
									}
								} else {
									tempData[h].j = j;
								}

								lastHeights[k] += data->at(i).data.at(j).quantity;
								break;
							}
						}
					}
				}
			}

			for (int h=0; h<tempData.size(); h++) {
				if (-1 < tempData.at(h).j && tempData.at(h).j < data->at(tempData.at(h).i).data.size()) {
					tempData[h].j++;
				}
			}
		}
	}
}

void StackedGraph::paintStackedGraph()
{
	if (!data->isEmpty()) {
		calculateXYScale();
		int graphLength = getXPos(lastYear, lastMonth);
		QVector<double> lastHeights;
		for(int i=0; i<=graphLength; i++) {
			lastHeights.push_back(0.0);
		}

		if (baselineAlgo == 1) {
			for (int k=0; k<lastHeights.size(); k++) {

				for (int i=0; i<tempData.size(); i++) {
					for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
						if (k == getDataXPos(tempData.at(i).i,j)) {
							lastHeights[k] += data->at(tempData.at(i).i).data.at(j).quantity;
							break;
						}
					}
				}

				lastHeights[k] = -lastHeights.at(k) / 2.0;
			}
		} else if (baselineAlgo == 2) {
			int baselinePos = 0;
			for (int i=0; i<tempData.size(); i++) {
				if (tempData.at(i).i == baselinePos) {
					baselinePos = i;
					break;
				}
			}
			for (int k=0; k<lastHeights.size(); k++) {

				if (k > getDataXPos(tempData.at(baselinePos).i, data->at(tempData.at(baselinePos).i).data.size()-1)) {
					for (int i=baselinePos+1; i<tempData.size(); i++) {
						int j;
						for (j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
							if (k == getDataXPos(tempData.at(i).i,j)) {
								baselinePos = i;
								break;
							}
						}
						if (j < data->at(tempData.at(i).i).data.size()) {
							break;
						}
					}
				}

				for (int i=0; i<=baselinePos; i++) {
					for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
						if (k == getDataXPos(tempData.at(i).i,j)) {
							if (i != baselinePos) {
								lastHeights[k] += data->at(tempData.at(i).i).data.at(j).quantity;
							} else {
								lastHeights[k] += data->at(tempData.at(i).i).data.at(j).quantity / 2.0;
							}
							break;
						}
					}
				}

				lastHeights[k] = -lastHeights.at(k);
			}
		}

		double yMinPos = DBL_MAX;
		for (int i=0; i<lastHeights.size(); i++) {
			if (yMinPos > lastHeights.at(i)) {
				yMinPos = lastHeights.at(i);
			}
		}

		for (int i=0; i<tempData.size(); i++) {
			QPainterPath path;
			for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {

				int pX = getDataXPos(tempData.at(i).i,j);
				double pY = data->at(tempData.at(i).i).data.at(j).quantity + lastHeights.at(pX);
				if (j==0) {
					resetPathTo(path, double(pX), pY);
				} else {
					pathTo(path, double(pX), pY);
				}
			}
			for (int j=data->at(tempData.at(i).i).data.size()-1; j>=0; j--) {
				int pX = getDataXPos(tempData.at(i).i,j);
				double pY = lastHeights.at(pX);
				pathTo(path, double(pX), pY);
				if (j==0) {
					pY = data->at(tempData.at(i).i).data.at(j).quantity + lastHeights.at(pX);
					pathTo(path, double(pX), pY);
					int k = textPosIndex(data->at(tempData.at(i).i).data);
					pX = getDataXPos(tempData.at(i).i,k);
					pY = (data->at(tempData.at(i).i).data.at(k).quantity / 2.3) + lastHeights.at(pX);
					double posCorrection;
					if (k == 0) {
						posCorrection = 0.0;
					} else if (k == data->at(tempData.at(i).i).data.size()-1) {
						posCorrection = 0.35;
					} else {
						posCorrection = 0.25;
					}
					textTo(path, double(pX)-posCorrection, pY, data->at(tempData.at(i).i).data.at(k).quantity, data->at(tempData.at(i).i).label);
				}
			}

			double val = 0.0;
			if (0.0 < colorAlgoMaxValue-colorAlgoMinValue) {
				double currentValue = 0.0;
				if (colorAlgo == 0) {
					currentValue = getDataXPos(tempData.at(i).i,0);
				} else if (colorAlgo == 1) {
					currentValue = getDataXPos(tempData.at(i).i,data->at(tempData.at(i).i).data.size()-1);
				} else if (colorAlgo == 2) {
					double currentMaxQuantity = 0.0;
					for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
						if (currentMaxQuantity < data->at(tempData.at(i).i).data.at(j).quantity) {
							currentMaxQuantity = data->at(tempData.at(i).i).data.at(j).quantity;
						}
					}
					currentValue = currentMaxQuantity;
				} else if (colorAlgo == 3) {
					double currentAverageQuantity = 0.0;
					for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
						currentAverageQuantity += data->at(tempData.at(i).i).data.at(j).quantity / double(data->at(tempData.at(i).i).data.size());
					}
					currentValue = currentAverageQuantity;
				} else if (colorAlgo == 4) {
					double currentSumQuantity = 0.0;
					for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
						currentSumQuantity += data->at(tempData.at(i).i).data.at(j).quantity;
					}
					currentValue = currentSumQuantity;
				}
				val = (currentValue - colorAlgoMinValue) / (colorAlgoMaxValue - colorAlgoMinValue);
			}

			tempData[i].item = (QGraphicsItem*)this->addPath(path,QPen(Qt::black, 0.5), colorScheme.getColor(val));

			for(int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
				int pX = getDataXPos(tempData.at(i).i,j);
				lastHeights[pX] += data->at(tempData.at(i).i).data.at(j).quantity;
			}
		}

		yMaxPos = DBL_MIN;
		for (int i=0; i<lastHeights.size(); i++) {
			if (yMaxPos < lastHeights.at(i)) {
				yMaxPos = lastHeights.at(i);
			}
		}
		yRange = (yMaxPos - yMinPos) * yScale;
	}
}

int StackedGraph::getDataXPos(int i, int j)
{
	if (-1 < i && i < data->size() && -1 < j && j < data->at(i).data.size()) {
		return getXPos(data->at(i).data.at(j).year, data->at(i).data.at(j).month);
	} else {
		return 0;
	}
}

int StackedGraph::getXPos(int year, int month)
{
	return (year - firstYear) * 12 + (month - firstMonth);
}

int StackedGraph::getAverageUnitSize(const QVector<DataItemPart>& items)
{
	QVector<int> unitSizes;
	for (int i=0; i<items.size(); i++) {
		double number = items.at(i).quantity;
		if (number > 0.0) {
			int j=0;
			for (j=0; j<unitSizeNames->size()-1; j++) {
				if (number >= 1000.0) {
					number = number / 1000.0;
				} else {
					break;
				}
			}
			unitSizes.push_back(j);
		}
	}
	double average = 0.0;
	for (int i=0; i<unitSizes.size(); i++) {
		average += double(unitSizes.at(i)) / double(unitSizes.size());
	}
	return floor(average+0.5);
}

QString StackedGraph::addUnitToNumber(double number, int unitSize)
{
	int i=0;
	if (!detailedInformation) {
		if (unitSize < 0) {
			for (i=0; i<unitSizeNames->size()-1; i++) {
				if (number >= 1000.0) {
					number = number / 1000.0;
				} else {
					break;
				}
			}
		} else {
			unitSize++;
			if (unitSize > unitSizeNames->size()) {
				unitSize = unitSizeNames->size();
			}
			for (i=0; i<unitSize-1; i++) {
				number = number / 1000.0;
			}
		}
	}
	int precision = 0;
	if (0.0 < number) {
		if (number < 0.0095) {
			precision = 3;
		} else if (number < 0.095) {
			precision = 2;
		} else if (number < 0.95) {
			precision = 1;
		}
	}
	QString text = QString::number(number, 'f', precision);
	if (i < unitSizeNames->size()) {
		text += unitSizeNames->at(i);
	}
	if (*unitAfterNumber) {
		text += *unitSymbol;
	} else {
		text = *unitSymbol + text;
	}
	return text;
}

void StackedGraph::pathTo(QPainterPath& path, double pX, double pY)
{
	double cX1 = oX + ((pX - oX) / 2.0);
	double cY1 = oY;
	double cX2 = cX1;
	double cY2 = pY;
	path.cubicTo(cX1 * xScale, -cY1 * yScale, cX2 * xScale, -cY2 * yScale, pX * xScale, -pY * yScale);
	oX = pX;
	oY = pY;
}

void StackedGraph::resetPathTo(QPainterPath& path, double pX, double pY)
{
	path.moveTo(pX * xScale, -pY * yScale);
	oX = pX;
	oY = pY;
}

void StackedGraph::textTo(QPainterPath& path, double pX, double pY, double quantity, const QString& label)
{
	QFont font;
	font.setPixelSize(xScale / 20.0 + ((quantity * yScale) / 20.0));
	QString text;
	if (13 < label.size()) {
		text = label.left(10) + "...";
	} else {
		text = label;
	}
	path.addText(pX * xScale, -pY * yScale, font, text);
}

int StackedGraph::textPosIndex(const QVector<DataItemPart>& data)
{
	double maxQuant = -1.0;
	for (int i=0; i<data.size(); i++) {
		if (data.at(i).quantity > (maxQuant * 1.5)) {
			maxQuant = data.at(i).quantity;
			if (i == data.size()-1) {
				return i;
			}
		} else {
			return i-1;
		}
	}
	return 0;
}

void StackedGraph::makeToolTip(QGraphicsItem* item, QPoint pos)
{
	if (item != NULL) {
		for (int i=0; i<tempData.size(); i++) {
			if (tempData.at(i).item == item) {
				int unitSize = getAverageUnitSize(data->at(tempData.at(i).i).data);
				QString text = data->at(tempData.at(i).i).label;
				text += "\n";
				double sumQuantity = 0.0;
				for (int j=0; j<data->at(tempData.at(i).i).data.size(); j++) {
					text += "\n";
					text += QString::number(data->at(tempData.at(i).i).data.at(j).year);
					text += ".";
					text += QString::number(data->at(tempData.at(i).i).data.at(j).month);
					text += ":	";
					text += addUnitToNumber(data->at(tempData.at(i).i).data.at(j).quantity, unitSize);
					sumQuantity += data->at(tempData.at(i).i).data.at(j).quantity;
				}
				text += "\n\nTotal:	";
				text += addUnitToNumber(sumQuantity, unitSize);

				QToolTip::showText(pos,text,view);
				break;
			}
		}
	}
}
