#ifndef STACKEDGRAPH_H
#define STACKEDGRAPH_H

#include <QGraphicsScene>
#include "StackedGraphView.h"
#include "DataItem.h"
#include "ColorScheme1D.h"

/**
* This structure represents a data item in the original data of the centralWidget. it is used for sorting.
*/
struct TempData
{
int i; ///< the position of the data item in data
int j; ///< the position of the data item part in data (just used while sorting)
QGraphicsItem* item; ///< the pointer to the QGraphicsItem which represents this data item
};

/**
* This class is a stacked graph.
*/
class StackedGraph : public QGraphicsScene
{
	Q_OBJECT

public:
	/**
	* \brief creates a stacked graph.
	* \param dataPtr the data which should be used for the stacked graph.
	* \param unitSymbolPtr the unit symbol for the data.
	* \param unitAfterNumberPtr if the unit symbol should be shown before or after the number.
	* \param unitSizeNamesPtr the size names for each 1000 step.
	* \param parent the parent widget.
	*/
	StackedGraph(const QVector<DataItem>* dataPtr, const QString* unitSymbolPtr, const bool* unitAfterNumberPtr, const QVector<QString>* unitSizeNamesPtr, QWidget* parent);
	~StackedGraph();

	StackedGraphView* getView() {return view;} ///< returns the view to this stacked graph
	int getLength() {calculateFirstLastDate(); return getXPos(lastYear, lastMonth);} ///< returns the length of the graph in months-1
	int getXRange() {return xRange;} ///< returns the x range (length)
	int getYRange() {return yRange;} ///< returns the y range (height)
	bool getRenderGraph() {return renderGraph;} ///< returns if the graph will be rendered on makeStackedGraph(). if false the graph won't be rendered if makeStackedGraph() is called.
	void setRenderGraph(bool renderGraphOn) {renderGraph = renderGraphOn;} ///< sets the renderGraph option. if false the graph won't be rendered if makeStackedGraph() is called.

signals:
	void maxColorTextChanged(const QString& text, const QColor& color); ///< this signal is send when the graph is rendered with the text and color for the max color button
	void minColorTextChanged(const QString& text, const QColor& color); ///< this signal is send when the graph is rendered with the text and color for the min color button

public slots:
	void makeStackedGraph(); ///< rebuilds the stacked graph
	void setXRange(int newXRange) {if (xRange != double(newXRange)) {xRange = double(newXRange); makeStackedGraph();}} ///< sets the xRange and rebuilds the graph
	void setBaselineAlgo(int newBaselineAlgo) {if (baselineAlgo != newBaselineAlgo) {baselineAlgo = newBaselineAlgo; makeStackedGraph();}} ///< sets the baseline algorithm and rebuilds the graph
	void setSortAlgo(int newSortAlgo) {if (sortAlgo != newSortAlgo) {sortAlgo = newSortAlgo; makeStackedGraph();}} ///< sets the sort algorithm and rebuilds the graph
	void setColorAlgo(int newColorAlgo) {if (colorAlgo != newColorAlgo) {colorAlgo = newColorAlgo; makeStackedGraph();}} ///< sets the color algorithm and rebuilds the graph
	void setMaxColor(const QColor& newMaxColor) {colorScheme.setColor_1(newMaxColor); makeStackedGraph();} ///< sets the max color and rebuilds the graph
	void setMinColor(const QColor& newMinColor) {colorScheme.setColor_2(newMinColor); makeStackedGraph();} ///< sets the min color and rebuilds the graph
	void setDetailedInformation(bool detailedInfo) {detailedInformation = detailedInfo; emitColorAlgoMinMaxValue();} ///< sets the detailed information and resends the min/max values
	void makeToolTip(QGraphicsItem* item, QPoint pos); ///< makes and shows the tool tip of the item item at the position pos

private:
	void calculateFirstLastDate(); ///< calculates firstYear,firstMonth,lastYear and lastMonth from the data data
	void calculateXYScale(); ///< calculates xScale and yScale from the data data
	void calculateColorAlgoMinMaxValue(); ///< calculates min and max value of the current set color algorithm from the data data
	void emitColorAlgoMinMaxValue(); ///< sends the maxColorTextChanged and minColorTextChanged signal
	void sortStackedGraph(); ///< sorts the stacked graph according to the sort algorithm
	void paintStackedGraph(); ///< paints the stacked graph according to the baseline and color algorithm
	int getDataXPos(int i, int j); ///< returns the x position from the given data item part at the position i and j from data. the position returned is months after the first month.
	int getXPos(int year, int month); ///< returns the x position from the year and month. the position returned is months after the first month.
	int getAverageUnitSize(const QVector<DataItemPart>& items); ///< returns the average unit size (successive division by 1000) from all data item parts in items.
	/**
	* \brief returns a string of the number with the unit and unit size added to the number.
	* \param number the number which should be turned into a string.
	* \param unitSize if unitSize is set to -1 it will automatically determine the unit size. if for example unitSize is set to 1 it means the number is divided by 1000, if set to 2 it is divided by 1000000.
	* \return a string of the number with the unit and unit size added to the number.
	*/
	QString addUnitToNumber(double number, int unitSize = -1);
	void pathTo(QPainterPath& path, double pX, double pY); ///< makes a cubic path from the last position (last time pathTo() was used) to pX and pY.
	void resetPathTo(QPainterPath& path, double pX, double pY); ///< sets or resets the path start to pX and pY.
	void textTo(QPainterPath& path, double pX, double pY, double quantity, const QString& label); ///< sets the text label to pX and pY.
	int textPosIndex(const QVector<DataItemPart>& data); ///< returns the vector position of the best position for the text of the given data item parts.

	StackedGraphView* view;
	const QVector<DataItem>* data;
	const QString* unitSymbol;
	const bool* unitAfterNumber;
	const QVector<QString>* unitSizeNames;
	QVector<TempData> tempData;
	int baselineAlgo,sortAlgo,colorAlgo;
	double oX,oY;
	double xScale,yScale;
	double xRange,yRange;
	double yMaxPos;
	double colorAlgoMinValue,colorAlgoMaxValue;
	int firstYear,firstMonth,lastYear,lastMonth;
	bool renderGraph;
	bool detailedInformation;
	ColorScheme1D colorScheme;
};

#endif
