#include "StackedGraphView.h"
#include "StackedGraph.h"
#include <QWheelEvent>
#include <QGLWidget>
#include <QTimer>
#include <math.h>

StackedGraphView::StackedGraphView(QGraphicsScene* scene, QWidget* parent) : QGraphicsView(scene,parent)
{
	this->setRenderHint(QPainter::Antialiasing);
	this->setRenderHint(QPainter::HighQualityAntialiasing);
	this->setBackgroundBrush(Qt::white);
	this->setDragMode(QGraphicsView::ScrollHandDrag);
	this->setMinimumSize(480, 300);
	hasGLSampleBuffer = false;
	hasGLViewport = false;
	autoGLViewport = false;
	pixelScale = 1.0;
	if (QGLFormat::hasOpenGL()) {
		QGLWidget* glWidget = new QGLWidget(QGLFormat(QGL::SampleBuffers));
		if (glWidget->format().sampleBuffers()) {
			hasGLSampleBuffer = true;
		}
		delete glWidget;
	}
}

StackedGraphView::~StackedGraphView()
{

}

void StackedGraphView::wheelEvent(QWheelEvent* event)
{
	if (event->orientation() == Qt::Vertical) {
		int degreesMoved = event->delta() / 8;
		double scale = 1.0 + (double(degreesMoved) / 100.0);
		this->scale(scale,scale);
		if (QGLFormat::hasOpenGL()) {
			pixelScale *= scale;
			checkGLViewport();
		}
	}
}

void StackedGraphView::mouseReleaseEvent(QMouseEvent* event)
{
	if (event->button() == Qt::LeftButton) {
		this->viewport()->setCursor(Qt::OpenHandCursor);
	} else if (event->button() == Qt::RightButton) {
		if (QGraphicsItem* item = this->itemAt(event->pos())) {
			emit toolTipChanged(item,event->globalPos());
		}
	} else if (event->button() == Qt::MidButton) {
		showWholeGraph();
	}
}

void StackedGraphView::showWholeGraph()
{
	resetView();
	QTimer::singleShot(0, this, SLOT(resetView()));
	QTimer::singleShot(0, this, SLOT(resetPixelScale()));
}

void StackedGraphView::resetPixelScale()
{
	if (QGLFormat::hasOpenGL() && hasGLSampleBuffer) {
		double ratioView = double(this->viewport()->width()) / double(this->viewport()->height());
		double ratioGraph = this->sceneRect().width() / this->sceneRect().height();
		double widthView = 0.0;
		double heightView = 0.0;
		if (ratioView <= ratioGraph) {
			widthView = double(this->viewport()->width());
			heightView = widthView / ratioGraph;
		} else {
			heightView = double(this->viewport()->height());
			widthView = heightView * ratioGraph;
		}
		StackedGraph* graph = (StackedGraph*)this->scene();
		double scaleGraphLength = double(graph->getXRange()) / (320.0 * double(graph->getLength()));
		pixelScale = (sqrt(pow(widthView,2) + pow(heightView,2)) * scaleGraphLength) / (400.0 * ratioGraph);
		checkGLViewport();
	}
}

void StackedGraphView::setGLViewport(bool openGL)
{
	if (QGLFormat::hasOpenGL() && hasGLSampleBuffer) {
		if (openGL) {
			this->setViewport(new QGLWidget(QGLFormat(QGL::SampleBuffers)));
			hasGLViewport = true;
		} else {
			this->setViewport(NULL);
			hasGLViewport = false;
		}
	}
}


void StackedGraphView::checkGLViewport()
{
	if (QGLFormat::hasOpenGL() && hasGLSampleBuffer && autoGLViewport) {
		if (!hasGLViewport && pixelScale > 4.0) {
			setGLViewport(true);
		} else if (hasGLViewport && pixelScale <= 4.0) {
			setGLViewport(false);
		}
	}
}
